"""Selects time slice"""

import logging
from typing import Union

import xarray as xr
from torch.utils.data import IterDataPipe, functional_datapipe

logger = logging.getLogger(__name__)


@functional_datapipe("select_id")
class SelectIDIterDataPipe(IterDataPipe):
    """Selects time slice"""

    def __init__(
        self,
        source_datapipe: IterDataPipe,
        location_datapipe: IterDataPipe,
        data_source_name: str = "nwp",
    ):
        """
        Selects id

        Args:
            source_datapipe: Datapipe of Xarray objects
            location_datapipe: Location datapipe
            data_source_name: the name of the datasource.
        """
        self.source_datapipe = source_datapipe
        self.location_datapipe = location_datapipe
        self.data_source_name = data_source_name

        assert data_source_name in ["pv"]

    def __iter__(self) -> Union[xr.DataArray, xr.Dataset]:
        for xr_data, location in self.source_datapipe.zip_ocf(self.location_datapipe):
            logger.debug(f"Selecting Data on id {location.id} for {self.data_source_name}")

            if self.data_source_name == "pv":
                try:
                    xr_data = xr_data.sel(pv_system_id=[location.id])
                except Exception as e:
                    logger.warning(
                        f"Failed using [{location.id}], now trying without brackets, error: {e}"
                    )
                    try:
                        # TODO Fix this, simple ones fail without the [], MetNet fails with []
                        xr_data = xr_data.sel(pv_system_id=location.id)
                    except Exception as e:
                        logger.warning(f"Could not find {location.id} in pv {xr_data.pv_system_id}")
                        raise e
            logger.debug("Selected Data on id")
            yield xr_data



"""Convert PV to Numpy Batch"""

import logging

import numpy as np
from torch.utils.data import IterDataPipe, functional_datapipe

from ocf_datapipes.batch import BatchKey, NumpyBatch
from ocf_datapipes.utils.utils import datetime64_to_float

logger = logging.getLogger(__name__)


def convert_pv_to_numpy_batch(xr_data):
    """Convert PV Xarray to NumpyBatch"""
    example: NumpyBatch = {
        BatchKey.pv: xr_data.values,
        BatchKey.pv_t0_idx: xr_data.attrs["t0_idx"],
        BatchKey.pv_ml_id: xr_data["ml_id"].values,
        BatchKey.pv_id: xr_data["pv_system_id"].values.astype(np.float32),
        BatchKey.pv_observed_capacity_wp: (xr_data["observed_capacity_wp"].values),
        BatchKey.pv_nominal_capacity_wp: (xr_data["nominal_capacity_wp"].values),
        BatchKey.pv_time_utc: datetime64_to_float(xr_data["time_utc"].values),
        BatchKey.pv_latitude: xr_data["latitude"].values,
        BatchKey.pv_longitude: xr_data["longitude"].values,
    }

    return example


@functional_datapipe("convert_pv_to_numpy_batch")
class ConvertPVToNumpyBatchIterDataPipe(IterDataPipe):
    """Convert PV Xarray to NumpyBatch"""

    def __init__(self, source_datapipe: IterDataPipe):
        """
        Convert PV Xarray objects to NumpyBatch objects

        Args:
            source_datapipe: Datapipe emitting PV Xarray objects
        """
        super().__init__()
        self.source_datapipe = source_datapipe

    def __iter__(self) -> NumpyBatch:
        """Convert PV Xarray to NumpyBatch"""
        for xr_data in self.source_datapipe:
            yield convert_pv_to_numpy_batch(xr_data)
